@@ -11,17 +11,19 @@ use App::Wax;
 use Method::Signatures::Simple;
 use Test::Differences qw(eq_or_diff);
 use Test::TinyMocker qw(mock);
+use URI::Split qw(uri_split);
 
 my @FILENAMES = ('1.json', '2.html');
 
-our @KEEP = map { "/cache/file$_" } @FILENAMES;
+our @DEFAULT = map { "/home/test/.cache/wax/file$_" } @FILENAMES;
+our @KEEP = map { "/keep/file$_" } @FILENAMES;
 our @TEMP = map { "/tmp/file$_" } @FILENAMES;
 our @URL = map { "https://example.com/$_" } @FILENAMES;
 
 my %FILENAME_TEMP = map { $URL[$_] => $TEMP[$_] } 0 .. $#FILENAMES;
 my %FILENAME_KEEP = map { $URL[$_] => $KEEP[$_] } 0 .. $#FILENAMES;
 
-our @EXPORT_OK = qw(@KEEP @TEMP @URL wax_is);
+our @EXPORT_OK = qw(@DEFAULT @KEEP @TEMP @URL wax_is);
 
 # a test helper (assertion) which takes a wax command (string/arrayref) and the
 # command we expect it to be translated into (string/arrayref) then calls wax
@@ -40,7 +42,9 @@ func wax_is ($args, $want) {
         $wax->dump_command(\@want)
     );
 
-    my $got = $wax->run([ '--test', @args ]);
+    local $ENV{XDG_CACHE_HOME} = '/home/test/.cache';
+
+    my $got = $wax->run(['--test', @args]);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     eq_or_diff $got, \@want, $description;
@@ -53,11 +57,25 @@ func wax_is ($args, $want) {
 mock(
     'App::Wax::resolve' => method ($_url) {
         my ($url, $url_index) = @$_url;
-        my $filename = $self->keep ? $FILENAME_KEEP{$url} : $FILENAME_TEMP{$url};
+        my $dir = $self->directory;
+        my $filename;
+
+        if ($dir) {
+            my ($scheme, $domain, $path, $query, $fragment) = uri_split($url);
+            $path =~ s{^/}{};
+            $filename = "$dir/file$path";
+        } else {
+            $filename = $self->keep ? $FILENAME_KEEP{$url} : $FILENAME_TEMP{$url};
+        }
+
         my @resolved = ($filename, undef); # (filename, error)
 
         return wantarray ? @resolved : \@resolved;
-    }
+    },
 );
 
+# don't check/create the directory specified by --directory or implied by
+# --default-directory
+mock('App::Wax::_check_directory' => sub {});
+
 1;
